"use strict";
// synchronous object signal for S.js
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lift = void 0;
const s_js_1 = __importDefault(require("s-js"));
// to get rid of the export default
const S = s_js_1.default;
const SObject = function (value) {
    const data = S(() => value);
    return lift(data, value);
};
exports.default = SObject;
function lift(data, value = S.sample(data)) {
    S.on(data, () => (value = data()));
    const values = new Map();
    S.on(data, updateValues);
    const valueProxy = new Proxy(value, {
        get(_target, p, receiver) {
            const signal = values.get(p);
            if (signal) {
                return signal();
            }
            return Reflect.get(value, p, receiver);
        },
        set(_target, p, value, receiver) {
            const signal = values.get(p);
            if (signal) {
                signal(value);
                return true;
            }
            return Reflect.set(value, p, value, receiver);
        },
        deleteProperty(_target, p) {
            const signal = values.get(p);
            if (signal) {
                signal(undefined);
            }
            return Reflect.deleteProperty(value, p);
        },
    });
    const object = Object.assign(function object(newValue) {
        if (arguments.length > 0) {
            return data(newValue);
        }
        else {
            return data();
        }
    }, { get, set, proxy: valueProxy });
    function get(key) {
        return getValueSignal(key)();
    }
    function set(key, newValue) {
        return getValueSignal(key)(newValue);
    }
    function updateValues() {
        const newData = data();
        for (const _key of Object.keys(newData)) {
            const key = _key;
            if (values.has(key)) {
                const signal = values.get(key);
                const newValue = newData[key];
                signal(newValue);
            }
            else {
                values.set(key, makeValueSignal(key));
            }
        }
    }
    function makeValueSignal(key) {
        const signal = S.value(data()[key]);
        S.on(signal, () => {
            S.sample(data)[key] = signal();
        });
        return signal;
    }
    function getValueSignal(key) {
        if (values.has(key)) {
            return values.get(key);
        }
        else {
            const signal = makeValueSignal(key);
            values.set(key, signal);
            return signal;
        }
    }
    return object;
}
exports.lift = lift;
//# sourceMappingURL=index.js.map